# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "PriceExtensionTypeEnum",
    },
)


class PriceExtensionTypeEnum(proto.Message):
    r"""Container for enum describing types for a price extension."""

    class PriceExtensionType(proto.Enum):
        r"""Price extension type.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            BRANDS (2):
                The type for showing a list of brands.
            EVENTS (3):
                The type for showing a list of events.
            LOCATIONS (4):
                The type for showing locations relevant to
                your business.
            NEIGHBORHOODS (5):
                The type for showing sub-regions or districts
                within a city or region.
            PRODUCT_CATEGORIES (6):
                The type for showing a collection of product
                categories.
            PRODUCT_TIERS (7):
                The type for showing a collection of related
                product tiers.
            SERVICES (8):
                The type for showing a collection of services
                offered by your business.
            SERVICE_CATEGORIES (9):
                The type for showing a collection of service
                categories.
            SERVICE_TIERS (10):
                The type for showing a collection of related
                service tiers.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        BRANDS = 2
        EVENTS = 3
        LOCATIONS = 4
        NEIGHBORHOODS = 5
        PRODUCT_CATEGORIES = 6
        PRODUCT_TIERS = 7
        SERVICES = 8
        SERVICE_CATEGORIES = 9
        SERVICE_TIERS = 10


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import linked_product_type
from google.ads.googleads.v20.enums.types import product_link_invitation_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "ProductLinkInvitation",
        "HotelCenterLinkInvitationIdentifier",
        "MerchantCenterLinkInvitationIdentifier",
        "AdvertisingPartnerLinkInvitationIdentifier",
    },
)


class ProductLinkInvitation(proto.Message):
    r"""Represents an invitation for data sharing connection between
    a Google Ads account and another account.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of a product link invitation.
            Product link invitation resource names have the form:

            ``customers/{customer_id}/productLinkInvitations/{product_link_invitation_id}``
        product_link_invitation_id (int):
            Output only. The ID of the product link
            invitation. This field is read only.
        status (google.ads.googleads.v20.enums.types.ProductLinkInvitationStatusEnum.ProductLinkInvitationStatus):
            Output only. The status of the product link
            invitation. This field is read only.
        type_ (google.ads.googleads.v20.enums.types.LinkedProductTypeEnum.LinkedProductType):
            Output only. The type of the invited account.
            This field is read only and can be used for
            filtering invitations with {@code
            GoogleAdsService.SearchGoogleAdsRequest}.
        hotel_center (google.ads.googleads.v20.resources.types.HotelCenterLinkInvitationIdentifier):
            Output only. Hotel link invitation.

            This field is a member of `oneof`_ ``invited_account``.
        merchant_center (google.ads.googleads.v20.resources.types.MerchantCenterLinkInvitationIdentifier):
            Output only. Merchant Center link invitation.

            This field is a member of `oneof`_ ``invited_account``.
        advertising_partner (google.ads.googleads.v20.resources.types.AdvertisingPartnerLinkInvitationIdentifier):
            Output only. Advertising Partner link
            invitation.

            This field is a member of `oneof`_ ``invited_account``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    product_link_invitation_id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    status: (
        product_link_invitation_status.ProductLinkInvitationStatusEnum.ProductLinkInvitationStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=product_link_invitation_status.ProductLinkInvitationStatusEnum.ProductLinkInvitationStatus,
    )
    type_: linked_product_type.LinkedProductTypeEnum.LinkedProductType = (
        proto.Field(
            proto.ENUM,
            number=6,
            enum=linked_product_type.LinkedProductTypeEnum.LinkedProductType,
        )
    )
    hotel_center: "HotelCenterLinkInvitationIdentifier" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="invited_account",
        message="HotelCenterLinkInvitationIdentifier",
    )
    merchant_center: "MerchantCenterLinkInvitationIdentifier" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="invited_account",
        message="MerchantCenterLinkInvitationIdentifier",
    )
    advertising_partner: "AdvertisingPartnerLinkInvitationIdentifier" = (
        proto.Field(
            proto.MESSAGE,
            number=7,
            oneof="invited_account",
            message="AdvertisingPartnerLinkInvitationIdentifier",
        )
    )


class HotelCenterLinkInvitationIdentifier(proto.Message):
    r"""The identifier for Hotel account.

    Attributes:
        hotel_center_id (int):
            Output only. The hotel center id of the hotel
            account. This field is read only
    """

    hotel_center_id: int = proto.Field(
        proto.INT64,
        number=1,
    )


class MerchantCenterLinkInvitationIdentifier(proto.Message):
    r"""The identifier for Merchant Center Account.

    Attributes:
        merchant_center_id (int):
            Output only. The Merchant Center id of the
            Merchant account. This field is read only
    """

    merchant_center_id: int = proto.Field(
        proto.INT64,
        number=1,
    )


class AdvertisingPartnerLinkInvitationIdentifier(proto.Message):
    r"""The identifier for the Advertising Partner Google Ads
    account.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer (str):
            Immutable. The resource name of the
            advertising partner Google Ads account. This
            field is read only.

            This field is a member of `oneof`_ ``_customer``.
    """

    customer: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    local_services_conversation_type,
)
from google.ads.googleads.v22.enums.types import local_services_participant_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "LocalServicesLeadConversation",
        "PhoneCallDetails",
        "MessageDetails",
    },
)


class LocalServicesLeadConversation(proto.Message):
    r"""Data from Local Services Lead Conversation.
    Contains details of Lead Conversation which is generated when
    user calls, messages or books service from advertiser. These are
    appended to a Lead. More info:
    https://ads.google.com/local-services-ads


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the local services lead
            conversation data. Local Services Lead Conversation resource
            name have the form

            ``customers/{customer_id}/localServicesLeadConversation/{local_services_lead_conversation_id}``
        id (int):
            Output only. ID of this Lead Conversation.
        conversation_channel (google.ads.googleads.v22.enums.types.LocalServicesLeadConversationTypeEnum.ConversationType):
            Output only. Type of GLS lead conversation, EMAIL, MESSAGE,
            PHONE_CALL, SMS, etc.
        participant_type (google.ads.googleads.v22.enums.types.LocalServicesParticipantTypeEnum.ParticipantType):
            Output only. Type of participant in the lead
            conversation, ADVERTISER or CONSUMER.
        lead (str):
            Output only. Resource name of Lead associated
            to the Lead Conversation.
        event_date_time (str):
            Output only. The date time at which lead
            conversation was created by Local Services Ads.
            The format is "YYYY-MM-DD HH:MM:SS" in the
            Google Ads account's timezone. Examples:
            "2018-03-05 09:15:00" or "2018-02-01 14:34:30".
        phone_call_details (google.ads.googleads.v22.resources.types.PhoneCallDetails):
            Output only. Details of phone call conversation in case of
            PHONE_CALL.

            This field is a member of `oneof`_ ``_phone_call_details``.
        message_details (google.ads.googleads.v22.resources.types.MessageDetails):
            Output only. Details of message conversation
            in case of EMAIL, MESSAGE or SMS.

            This field is a member of `oneof`_ ``_message_details``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    conversation_channel: (
        local_services_conversation_type.LocalServicesLeadConversationTypeEnum.ConversationType
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=local_services_conversation_type.LocalServicesLeadConversationTypeEnum.ConversationType,
    )
    participant_type: (
        local_services_participant_type.LocalServicesParticipantTypeEnum.ParticipantType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=local_services_participant_type.LocalServicesParticipantTypeEnum.ParticipantType,
    )
    lead: str = proto.Field(
        proto.STRING,
        number=5,
    )
    event_date_time: str = proto.Field(
        proto.STRING,
        number=6,
    )
    phone_call_details: "PhoneCallDetails" = proto.Field(
        proto.MESSAGE,
        number=7,
        optional=True,
        message="PhoneCallDetails",
    )
    message_details: "MessageDetails" = proto.Field(
        proto.MESSAGE,
        number=8,
        optional=True,
        message="MessageDetails",
    )


class PhoneCallDetails(proto.Message):
    r"""Represents details of a phone call conversation.

    Attributes:
        call_duration_millis (int):
            Output only. The duration (in milliseconds)
            of the phone call (end to end).
        call_recording_url (str):
            Output only. URL to the call recording audio
            file.
    """

    call_duration_millis: int = proto.Field(
        proto.INT64,
        number=1,
    )
    call_recording_url: str = proto.Field(
        proto.STRING,
        number=2,
    )


class MessageDetails(proto.Message):
    r"""Represents details of text message in case of email, message
    or SMS.

    Attributes:
        text (str):
            Output only. Textual content of the message.
        attachment_urls (MutableSequence[str]):
            Output only. URL to the SMS or email
            attachments. These URLs can be used to download
            the contents of the attachment by using the
            developer token.
    """

    text: str = proto.Field(
        proto.STRING,
        number=1,
    )
    attachment_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "BiddingStrategyErrorEnum",
    },
)


class BiddingStrategyErrorEnum(proto.Message):
    r"""Container for enum describing possible bidding strategy
    errors.

    """

    class BiddingStrategyError(proto.Enum):
        r"""Enum describing possible bidding strategy errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            DUPLICATE_NAME (2):
                Each bidding strategy must have a unique
                name.
            CANNOT_CHANGE_BIDDING_STRATEGY_TYPE (3):
                Bidding strategy type is immutable.
            CANNOT_REMOVE_ASSOCIATED_STRATEGY (4):
                Only bidding strategies not linked to
                campaigns, adgroups or adgroup criteria can be
                removed.
            BIDDING_STRATEGY_NOT_SUPPORTED (5):
                The specified bidding strategy is not
                supported.
            INCOMPATIBLE_BIDDING_STRATEGY_AND_BIDDING_STRATEGY_GOAL_TYPE (6):
                The bidding strategy is incompatible with the
                campaign's bidding strategy goal type.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DUPLICATE_NAME = 2
        CANNOT_CHANGE_BIDDING_STRATEGY_TYPE = 3
        CANNOT_REMOVE_ASSOCIATED_STRATEGY = 4
        BIDDING_STRATEGY_NOT_SUPPORTED = 5
        INCOMPATIBLE_BIDDING_STRATEGY_AND_BIDDING_STRATEGY_GOAL_TYPE = 6


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This code example adds two ad customizer attributes.

It then associates them with the given ad group and adds an ad that uses the ad
customizer attributes to populate dynamic data.
"""


import argparse
import sys
from uuid import uuid4

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types import AdTextAsset
from google.ads.googleads.v22.resources.types import (
    AdGroupAd,
    AdGroupCustomizer,
    CustomizerAttribute,
)
from google.ads.googleads.v22.services.services.ad_group_ad_service import (
    AdGroupAdServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_customizer_service import (
    AdGroupCustomizerServiceClient,
)
from google.ads.googleads.v22.services.services.customizer_attribute_service import (
    CustomizerAttributeServiceClient,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types import (
    AdGroupAdOperation,
    AdGroupCustomizerOperation,
    CustomizerAttributeOperation,
    MutateAdGroupAdsResponse,
    MutateAdGroupCustomizersResponse,
    MutateCustomizerAttributesResponse,
)


def main(client: GoogleAdsClient, customer_id: str, ad_group_id: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        ad_group_id: an ad group ID.
    """
    text_customizer_name: str = f"Planet_{uuid4().hex[:8]}"
    price_customizer_name: str = f"Price_{uuid4().hex[:8]}"

    text_customizer_resource_name: str = create_text_customizer_attribute(
        client, customer_id, text_customizer_name
    )
    price_customizer_resource_name: str = create_price_customizer_attribute(
        client, customer_id, price_customizer_name
    )
    link_customizer_attributes(
        client,
        customer_id,
        ad_group_id,
        text_customizer_resource_name,
        price_customizer_resource_name,
    )
    create_ad_with_customizations(
        client,
        customer_id,
        ad_group_id,
        text_customizer_name,
        price_customizer_name,
    )


# [START add_ad_customizer]
def create_text_customizer_attribute(
    client: GoogleAdsClient, customer_id: str, customizer_name: str
) -> str:
    """Creates a text customizer attribute and returns its resource name.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        customizer_name: the name of the customizer to create.

    Returns:
        a resource name for a text customizer attribute.
    """
    customizer_attribute_service: CustomizerAttributeServiceClient = (
        client.get_service("CustomizerAttributeService")
    )

    # Creates a text customizer attribute. The customizer attribute name is
    # arbitrary and will be used as a placeholder in the ad text fields.
    operation: CustomizerAttributeOperation = client.get_type(
        "CustomizerAttributeOperation"
    )
    text_attribute: CustomizerAttribute = operation.create
    text_attribute.name = customizer_name
    text_attribute.type_ = client.enums.CustomizerAttributeTypeEnum.TEXT

    response: MutateCustomizerAttributesResponse = (
        customizer_attribute_service.mutate_customizer_attributes(
            customer_id=customer_id, operations=[operation]
        )
    )

    resource_name: str = response.results[0].resource_name
    print(
        f"Added text customizer attribute with resource name '{resource_name}'"
    )
    return resource_name
    # [END add_ad_customizer]


# [START add_ad_customizer_1]
def create_price_customizer_attribute(
    client: GoogleAdsClient, customer_id: str, customizer_name: str
) -> str:
    """Creates a price customizer attribute and returns its resource name.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        customizer_name: the name of the customizer to create.

    Returns:
        a resource name for a text customizer attribute.
    """
    customizer_attribute_service: CustomizerAttributeServiceClient = (
        client.get_service("CustomizerAttributeService")
    )

    # Creates a price customizer attribute. The customizer attribute name is
    # arbitrary and will be used as a placeholder in the ad text fields.
    operation: CustomizerAttributeOperation = client.get_type(
        "CustomizerAttributeOperation"
    )
    price_attribute: CustomizerAttribute = operation.create
    price_attribute.name = customizer_name
    price_attribute.type_ = client.enums.CustomizerAttributeTypeEnum.PRICE

    response: MutateCustomizerAttributesResponse = (
        customizer_attribute_service.mutate_customizer_attributes(
            customer_id=customer_id, operations=[operation]
        )
    )

    resource_name: str = response.results[0].resource_name
    print(
        f"Added price customizer attribute with resource name '{resource_name}'"
    )
    return resource_name
    # [END add_ad_customizer_1]


# [START add_ad_customizer_2]
def link_customizer_attributes(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    text_customizer_resource_name: str,
    price_customizer_resource_name: str,
) -> None:
    """Restricts the ad customizer attributes to work with a specific ad group.

    This prevents the customizer attributes from being used elsewhere and makes
    sure they are used only for customizing a specific ad group.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        ad_group_id: the ad group ID to bind the customizer attributes to.
        text_customizer_resource_name: the resource name of the text customizer attribute.
        price_customizer_resource_name: the resource name of the price customizer attribute.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    ad_group_customizer_service: AdGroupCustomizerServiceClient = (
        client.get_service("AdGroupCustomizerService")
    )

    # Binds the text attribute customizer to a specific ad group to make sure
    # it will only be used to customize ads inside that ad group.
    mars_operation: AdGroupCustomizerOperation = client.get_type(
        "AdGroupCustomizerOperation"
    )
    mars_customizer: AdGroupCustomizer = mars_operation.create
    mars_customizer.customizer_attribute = text_customizer_resource_name
    mars_customizer.value.type_ = client.enums.CustomizerAttributeTypeEnum.TEXT
    mars_customizer.value.string_value = "Mars"
    mars_customizer.ad_group = googleads_service.ad_group_path(
        customer_id, ad_group_id
    )

    # Binds the price attribute customizer to a specific ad group to make sure
    # it will only be used to customize ads inside that ad group.
    price_operation: AdGroupCustomizerOperation = client.get_type(
        "AdGroupCustomizerOperation"
    )
    price_customizer: AdGroupCustomizer = price_operation.create
    price_customizer.customizer_attribute = price_customizer_resource_name
    price_customizer.value.type_ = (
        client.enums.CustomizerAttributeTypeEnum.PRICE
    )
    price_customizer.value.string_value = "100.0€"
    price_customizer.ad_group = googleads_service.ad_group_path(
        customer_id, ad_group_id
    )

    response: MutateAdGroupCustomizersResponse = (
        ad_group_customizer_service.mutate_ad_group_customizers(
            customer_id=customer_id,
            operations=[mars_operation, price_operation],
        )
    )

    for result in response.results:
        print(
            "Added an ad group customizer with resource name "
            f"'{result.resource_name}'"
        )
        # [END add_ad_customizer_2]


# [START add_ad_customizer_3]
def create_ad_with_customizations(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    text_customizer_name: str,
    price_customizer_name: str,
) -> None:
    """Creates a responsive search ad (RSA).

    The RSA uses the ad customizer attributes to populate the placeholders.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        ad_group_id: the ad group ID to bind the customizer attributes to.
        text_customizer_name: name of the text customizer.
        price_customizer_name: name of the price customizer.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    ad_group_ad_service: AdGroupAdServiceClient = client.get_service(
        "AdGroupAdService"
    )

    # Creates a responsive search ad using the attribute customizer names as
    # placeholders and default values to be used in case there are no attribute
    # customizer values.
    operation: AdGroupAdOperation = client.get_type("AdGroupAdOperation")
    ad_group_ad: AdGroupAd = operation.create
    ad_group_ad.ad.final_urls.append("https://www.example.com")
    ad_group_ad.ad_group = googleads_service.ad_group_path(
        customer_id, ad_group_id
    )

    headline_1: AdTextAsset = client.get_type("AdTextAsset")
    headline_1.text = (
        f"Luxury cruise to {{CUSTOMIZER.{text_customizer_name}:Venus}}"
    )
    headline_1.pinned_field = client.enums.ServedAssetFieldTypeEnum.HEADLINE_1

    headline_2: AdTextAsset = client.get_type("AdTextAsset")
    headline_2.text = f"Only {{CUSTOMIZER.{price_customizer_name}:10.0€}}"

    headline_3: AdTextAsset = client.get_type("AdTextAsset")
    headline_3.text = f"Cruise to {{CUSTOMIZER.{text_customizer_name}:Venus}} for {{CUSTOMIZER.{price_customizer_name}:10.0€}}"

    ad_group_ad.ad.responsive_search_ad.headlines.extend(
        [headline_1, headline_2, headline_3]
    )

    description_1: AdTextAsset = client.get_type("AdTextAsset")
    description_1.text = (
        f"Tickets are only {{CUSTOMIZER.{price_customizer_name}:10.0€}}!"
    )

    description_2: AdTextAsset = client.get_type("AdTextAsset")
    description_2.text = (
        f"Buy your tickets to {{CUSTOMIZER.{text_customizer_name}:Venus}} now!"
    )

    ad_group_ad.ad.responsive_search_ad.descriptions.extend(
        [description_1, description_2]
    )

    response: MutateAdGroupAdsResponse = (
        ad_group_ad_service.mutate_ad_group_ads(
            customer_id=customer_id, operations=[operation]
        )
    )
    resource_name: str = response.results[0].resource_name
    print(f"Added an ad with resource name '{resource_name}'")
    # [END add_ad_customizer_3]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "This code example adds two ad customizer attributes and "
            "associates them with the ad group. Then it adds an ad that uses "
            "the customizer attributes to populate dynamic data."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--ad_group_id",
        type=str,
        required=True,
        help="An ad group ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.ad_group_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "ConversionUploadErrorEnum",
    },
)


class ConversionUploadErrorEnum(proto.Message):
    r"""Container for enum describing possible conversion upload
    errors.

    """

    class ConversionUploadError(proto.Enum):
        r"""Enum describing possible conversion upload errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            TOO_MANY_CONVERSIONS_IN_REQUEST (2):
                Upload fewer than 2001 events in a single
                request.
            UNPARSEABLE_GCLID (3):
                The imported gclid could not be decoded.
            CONVERSION_PRECEDES_EVENT (42):
                The imported event has a ``conversion_date_time`` that
                precedes the click. Make sure your ``conversion_date_time``
                is correct and try again.
            EXPIRED_EVENT (43):
                The imported event can't be recorded because
                its click occurred before this conversion's
                click-through window. Make sure you import the
                most recent data.
            TOO_RECENT_EVENT (44):
                The click associated with the given
                identifier or iOS URL parameter occurred less
                than 6 hours ago. Retry after 6 hours have
                passed.
            EVENT_NOT_FOUND (45):
                The imported event could not be attributed to
                a click. This may be because the event did not
                come from a Google Ads campaign.
            UNAUTHORIZED_CUSTOMER (8):
                The click ID or call is associated with an
                Ads account you don't have access to. Make sure
                you import conversions for accounts managed by
                your manager account.
            TOO_RECENT_CONVERSION_ACTION (10):
                Can't import events to a conversion action
                that was just created. Try importing again in 6
                hours.
            CONVERSION_TRACKING_NOT_ENABLED_AT_IMPRESSION_TIME (11):
                At the time of the click, conversion tracking
                was not enabled in the effective conversion
                account of the click's Google Ads account.
            EXTERNAL_ATTRIBUTION_DATA_SET_FOR_NON_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION (12):
                The imported event includes external
                attribution data, but the conversion action
                isn't set up to use an external attribution
                model. Make sure the conversion action is
                correctly configured and try again.
            EXTERNAL_ATTRIBUTION_DATA_NOT_SET_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION (13):
                The conversion action is set up to use an
                external attribution model, but the imported
                event is missing data. Make sure imported events
                include the external attribution credit and all
                necessary fields.
            ORDER_ID_NOT_PERMITTED_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION (14):
                Order IDs can't be used for a conversion
                measured with an external attribution model.
                Make sure the conversion is correctly configured
                and imported events include only necessary data
                and try again.
            ORDER_ID_ALREADY_IN_USE (15):
                The imported event includes an order ID that
                was previously recorded, so the event was not
                processed.
            DUPLICATE_ORDER_ID (16):
                Imported events include multiple conversions
                with the same order ID and were not processed.
                Make sure order IDs are unique and try again.
            TOO_RECENT_CALL (17):
                Can't import calls that occurred less than 6
                hours ago. Try uploading again in 6 hours.
            EXPIRED_CALL (18):
                The call can't be recorded because it
                occurred before this conversion action's
                lookback window. Make sure your import is
                configured to get the most recent data.
            CALL_NOT_FOUND (19):
                The call or click leading to the imported
                event can't be found. Make sure your data source
                is set up to include correct identifiers.
            CONVERSION_PRECEDES_CALL (20):
                The call has a ``conversion_date_time`` that precedes the
                associated click. Make sure your ``conversion_date_time`` is
                correct.
            CONVERSION_TRACKING_NOT_ENABLED_AT_CALL_TIME (21):
                At the time of the imported call, conversion
                tracking was not enabled in the effective
                conversion account of the click's Google Ads
                account.
            UNPARSEABLE_CALLERS_PHONE_NUMBER (22):
                Make sure phone numbers are formatted as
                E.164 (+16502531234), International (+64 3-331
                6005), or US national number (6502531234).
            CLICK_CONVERSION_ALREADY_EXISTS (23):
                The imported event has the same click and
                ``conversion_date_time`` as an existing conversion. Use a
                unique ``conversion_date_time`` or order ID for each unique
                event and try again.
            CALL_CONVERSION_ALREADY_EXISTS (24):
                The imported call has the same ``conversion_date_time`` as
                an existing conversion. Make sure your
                ``conversion_date_time`` correctly configured and try again.
            DUPLICATE_CLICK_CONVERSION_IN_REQUEST (25):
                Multiple events have the same click and
                ``conversion_date_time``. Make sure your
                ``conversion_date_time`` is correctly configured and try
                again.
            DUPLICATE_CALL_CONVERSION_IN_REQUEST (26):
                Multiple events have the same call and
                ``conversion_date_time``. Make sure your
                ``conversion_date_time`` is correctly configured and try
                again.
            CUSTOM_VARIABLE_NOT_ENABLED (28):
                Enable the custom variable in your conversion
                settings and try again.
            CUSTOM_VARIABLE_VALUE_CONTAINS_PII (29):
                Can't import events with custom variables
                containing personally-identifiable information
                (PII). Remove these variables and try again.
            INVALID_CUSTOMER_FOR_CLICK (30):
                The click from the imported event is
                associated with a different Google Ads account.
                Make sure you're importing to the correct
                account.
            INVALID_CUSTOMER_FOR_CALL (31):
                The click from the call is associated with a different
                Google Ads account. Make sure you're importing to the
                correct account. Query
                conversion_tracking_setting.google_ads_conversion_customer
                on Customer to identify the correct account.
            CONVERSION_NOT_COMPLIANT_WITH_ATT_POLICY (32):
                The connversion can't be imported because the
                conversion source didn't comply with Apple App
                Transparency Tracking (ATT) policies or because
                the customer didn't consent to tracking.
            CLICK_NOT_FOUND (33):
                The email address or phone number for this
                event can't be matched to a click. This may be
                because it didn't come from a Google Ads
                campaign, and you can safely ignore this
                warning. If this includes more imported events
                than is expected, you may need to check your
                setup.
            INVALID_USER_IDENTIFIER (34):
                Make sure you hash user provided data using
                SHA-256 and ensure you are normalizing according
                to the guidelines.
            EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION_NOT_PERMITTED_WITH_USER_IDENTIFIER (35):
                User provided data can't be used with
                external attribution models. Use a different
                attribution model or omit user identifiers and
                try again.
            UNSUPPORTED_USER_IDENTIFIER (36):
                The provided user identifiers are not
                supported. Use only hashed email or phone number
                and try again.
            GBRAID_WBRAID_BOTH_SET (38):
                Can't use both gbraid and wbraid parameters.
                Use only 1 and try again.
            UNPARSEABLE_WBRAID (39):
                Can't parse event import data. Check if your
                wbraid parameter was not modified and try again.
            UNPARSEABLE_GBRAID (40):
                Can't parse event import data. Check if your
                gbraid parameter was not modified and try again.
            ONE_PER_CLICK_CONVERSION_ACTION_NOT_PERMITTED_WITH_BRAID (46):
                Conversion actions that use one-per-click
                counting can't be used with gbraid or wbraid
                parameters.
            CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCED_CONVERSIONS (47):
                Enhanced conversions can't be used for this
                account because of Google customer data
                policies. Contact your Google representative.
            CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS (48):
                Make sure you agree to the customer data processing terms in
                conversion settings and try again. You can check your
                setting by querying
                conversion_tracking_setting.accepted_customer_data_terms on
                Customer.
            ORDER_ID_CONTAINS_PII (49):
                Can't import events with order IDs containing
                personally-identifiable information (PII).
            CUSTOMER_NOT_ENABLED_ENHANCED_CONVERSIONS_FOR_LEADS (50):
                Make sure you've turned on enhanced conversions for leads in
                conversion settings and try again. You can check your
                setting by querying
                conversion_tracking_setting.enhanced_conversions_for_leads_enabled
                on Customer.
            INVALID_JOB_ID (52):
                The provided job id in the request is not within the allowed
                range. A job ID must be a positive integer in the range [1,
                2^31).
            NO_CONVERSION_ACTION_FOUND (53):
                The conversion action specified in the upload
                request cannot be found. Make sure it's
                available in this account.
            INVALID_CONVERSION_ACTION_TYPE (54):
                The conversion action specified in the upload
                request isn't set up for uploading conversions.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        TOO_MANY_CONVERSIONS_IN_REQUEST = 2
        UNPARSEABLE_GCLID = 3
        CONVERSION_PRECEDES_EVENT = 42
        EXPIRED_EVENT = 43
        TOO_RECENT_EVENT = 44
        EVENT_NOT_FOUND = 45
        UNAUTHORIZED_CUSTOMER = 8
        TOO_RECENT_CONVERSION_ACTION = 10
        CONVERSION_TRACKING_NOT_ENABLED_AT_IMPRESSION_TIME = 11
        EXTERNAL_ATTRIBUTION_DATA_SET_FOR_NON_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION = (
            12
        )
        EXTERNAL_ATTRIBUTION_DATA_NOT_SET_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION = (
            13
        )
        ORDER_ID_NOT_PERMITTED_FOR_EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION = 14
        ORDER_ID_ALREADY_IN_USE = 15
        DUPLICATE_ORDER_ID = 16
        TOO_RECENT_CALL = 17
        EXPIRED_CALL = 18
        CALL_NOT_FOUND = 19
        CONVERSION_PRECEDES_CALL = 20
        CONVERSION_TRACKING_NOT_ENABLED_AT_CALL_TIME = 21
        UNPARSEABLE_CALLERS_PHONE_NUMBER = 22
        CLICK_CONVERSION_ALREADY_EXISTS = 23
        CALL_CONVERSION_ALREADY_EXISTS = 24
        DUPLICATE_CLICK_CONVERSION_IN_REQUEST = 25
        DUPLICATE_CALL_CONVERSION_IN_REQUEST = 26
        CUSTOM_VARIABLE_NOT_ENABLED = 28
        CUSTOM_VARIABLE_VALUE_CONTAINS_PII = 29
        INVALID_CUSTOMER_FOR_CLICK = 30
        INVALID_CUSTOMER_FOR_CALL = 31
        CONVERSION_NOT_COMPLIANT_WITH_ATT_POLICY = 32
        CLICK_NOT_FOUND = 33
        INVALID_USER_IDENTIFIER = 34
        EXTERNALLY_ATTRIBUTED_CONVERSION_ACTION_NOT_PERMITTED_WITH_USER_IDENTIFIER = (
            35
        )
        UNSUPPORTED_USER_IDENTIFIER = 36
        GBRAID_WBRAID_BOTH_SET = 38
        UNPARSEABLE_WBRAID = 39
        UNPARSEABLE_GBRAID = 40
        ONE_PER_CLICK_CONVERSION_ACTION_NOT_PERMITTED_WITH_BRAID = 46
        CUSTOMER_DATA_POLICY_PROHIBITS_ENHANCED_CONVERSIONS = 47
        CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS = 48
        ORDER_ID_CONTAINS_PII = 49
        CUSTOMER_NOT_ENABLED_ENHANCED_CONVERSIONS_FOR_LEADS = 50
        INVALID_JOB_ID = 52
        NO_CONVERSION_ACTION_FOUND = 53
        INVALID_CONVERSION_ACTION_TYPE = 54


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adds a page feed with URLs for a Dynamic Search Ads campaign."""

import argparse
import sys
from typing import List, Optional

from examples.utils.example_helpers import get_printable_datetime
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.asset_types import PageFeedAsset
from google.ads.googleads.v22.common.types.criteria import WebpageConditionInfo
from google.ads.googleads.v22.resources.types.ad_group_criterion import (
    AdGroupCriterion,
)
from google.ads.googleads.v22.resources.types.asset import Asset
from google.ads.googleads.v22.resources.types.asset_set import AssetSet
from google.ads.googleads.v22.resources.types.asset_set_asset import (
    AssetSetAsset,
)
from google.ads.googleads.v22.resources.types.campaign_asset_set import (
    CampaignAssetSet,
)
from google.ads.googleads.v22.services.services.ad_group_criterion_service import (
    AdGroupCriterionServiceClient,
)
from google.ads.googleads.v22.services.services.asset_service import (
    AssetServiceClient,
)
from google.ads.googleads.v22.services.services.asset_set_asset_service import (
    AssetSetAssetServiceClient,
)
from google.ads.googleads.v22.services.services.asset_set_service import (
    AssetSetServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_asset_set_service import (
    CampaignAssetSetServiceClient,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_criterion_service import (
    AdGroupCriterionOperation,
    MutateAdGroupCriteriaResponse,
)
from google.ads.googleads.v22.services.types.asset_service import (
    AssetOperation,
    MutateAssetResult,
    MutateAssetsResponse,
)
from google.ads.googleads.v22.services.types.asset_set_asset_service import (
    AssetSetAssetOperation,
    MutateAssetSetAssetResult,
    MutateAssetSetAssetsResponse,
)
from google.ads.googleads.v22.services.types.asset_set_service import (
    AssetSetOperation,
    MutateAssetSetsResponse,
)
from google.ads.googleads.v22.services.types.campaign_asset_set_service import (
    CampaignAssetSetOperation,
    MutateCampaignAssetSetsResponse,
)


# The label for the DSA page URLs.
DSA_PAGE_URL_LABEL = "discounts"


def main(
    client: GoogleAdsClient,
    customer_id: str,
    campaign_id: str,
    ad_group_id: Optional[str],
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        campaign_id: the ID for a Campaign.
        ad_group_id: the ID for an Ad Group.
    """
    # Creates assets.
    asset_resource_names: List[str] = create_assets(
        client, customer_id, DSA_PAGE_URL_LABEL
    )

    # Creates an asset set - this is a collection of assets that can be
    # associated with a campaign.
    # Note: do not confuse this with an asset group. An asset group replaces ad
    # groups in some types of campaigns.
    asset_set_resource_name: str = create_asset_set(client, customer_id)

    # Adds the assets to the asset set
    add_assets_to_asset_set(
        client, customer_id, asset_resource_names, asset_set_resource_name
    )

    # Links the asset set to the specified campaign.
    link_asset_set_to_campaign(
        client, customer_id, campaign_id, asset_set_resource_name
    )

    # Optional: Targets web pages matching the feed's label in the ad group.
    if ad_group_id:
        add_dsa_target(client, customer_id, ad_group_id, DSA_PAGE_URL_LABEL)


# [START add_asset]
def create_assets(
    client: GoogleAdsClient, customer_id: str, dsa_page_url_label: str
) -> List[str]:
    """Creates assets to be used in a DSA page feed.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        dsa_page_url_label: the label for the DSA page URLs.

    Returns:
        a list of the created assets' resource names.
    """
    urls: List[str] = [
        "http://www.example.com/discounts/rental-cars",
        "http://www.example.com/discounts/hotel-deals",
        "http://www.example.com/discounts/flight-deals",
    ]
    operations: List[AssetOperation] = []

    # Creates one asset per URL.
    for url in urls:
        # Creates an asset operation and adds it to the list of operations.
        operation: AssetOperation = client.get_type("AssetOperation")
        asset: Asset = operation.create
        page_feed_asset: PageFeedAsset = asset.page_feed_asset
        page_feed_asset.page_url = url
        # Recommended: adds labels to the asset. These labels can be used later
        # in ad group targeting to restrict the set of pages that can serve.
        page_feed_asset.labels.append(dsa_page_url_label)
        operations.append(operation)

    # Issues a mutate request to add the assets and prints its information.
    asset_service: AssetServiceClient = client.get_service("AssetService")
    response: MutateAssetsResponse = asset_service.mutate_assets(
        customer_id=customer_id, operations=operations
    )

    print(f"Added {len(response.results)} assets:")

    resource_names: List[str] = []
    result: MutateAssetResult
    for result in response.results:
        resource_name: str = result.resource_name
        print(f"\tCreated an asset with resource name: '{resource_name}'")
        resource_names.append(resource_name)

    return resource_names
    # [END add_asset]


# [START add_asset_set]
def create_asset_set(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates an asset set.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        the created asset set's resource name.
    """
    operation: AssetSetOperation = client.get_type("AssetSetOperation")
    # Creates an asset set which will be used to link the dynamic page feed
    # assets to a campaign.
    asset_set: AssetSet = operation.create
    asset_set.name = f"My dynamic page feed {get_printable_datetime()}"
    asset_set.type_ = client.enums.AssetSetTypeEnum.PAGE_FEED

    # Issues a mutate request to add the asset set and prints its information.
    asset_set_service: AssetSetServiceClient = client.get_service(
        "AssetSetService"
    )
    response: MutateAssetSetsResponse = asset_set_service.mutate_asset_sets(
        customer_id=customer_id, operations=[operation]
    )

    resource_name: str = response.results[0].resource_name
    print(f"Created an asset set with resource name: '{resource_name}'")
    return resource_name
    # [END add_asset_set]


# [START add_asset_set_asset]
def add_assets_to_asset_set(
    client: GoogleAdsClient,
    customer_id: str,
    asset_resource_names: List[str],
    asset_set_resource_name: str,
) -> None:
    """Adds assets to an asset set by creating an asset set asset link.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        asset_resource_names: a list of asset resource names.
        asset_set_resource_name: a resource name for an asset set.
    """
    operations: List[AssetSetAssetOperation] = []
    for resource_name in asset_resource_names:
        # Creates an asset set asset operation and adds it to the list of
        # operations.
        operation: AssetSetAssetOperation = client.get_type(
            "AssetSetAssetOperation"
        )
        asset_set_asset: AssetSetAsset = operation.create
        asset_set_asset.asset = resource_name
        asset_set_asset.asset_set = asset_set_resource_name
        operations.append(operation)

    # Issues a mutate request to add the asset set assets and prints its
    # information.
    asset_set_asset_service: AssetSetAssetServiceClient = client.get_service(
        "AssetSetAssetService"
    )
    response: MutateAssetSetAssetsResponse = (
        asset_set_asset_service.mutate_asset_set_assets(
            customer_id=customer_id, operations=operations
        )
    )

    print(f"Added {len(response.results)} asset set assets:")

    result: MutateAssetSetAssetResult
    for result in response.results:
        print(
            "\tCreated an asset set asset link with resource name "
            f"'{result.resource_name}'"
        )
        # [END add_asset_set_asset]


# [START add_campaign_asset_set]
def link_asset_set_to_campaign(
    client: GoogleAdsClient,
    customer_id: str,
    campaign_id: str,
    asset_set_resource_name: str,
) -> None:
    """Links the asset set to the campaign by creating a campaign asset set.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        campaign_id: the ID for a Campaign.
        asset_set_resource_name: a resource name for an asset set.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    # Creates a campaign asset set representing the link between an asset set
    # and a campaign.
    operation: CampaignAssetSetOperation = client.get_type(
        "CampaignAssetSetOperation"
    )
    campaign_asset_set: CampaignAssetSet = operation.create
    campaign_asset_set.asset_set = asset_set_resource_name
    campaign_asset_set.campaign = googleads_service.campaign_path(
        customer_id, campaign_id
    )

    campaign_asset_set_service: CampaignAssetSetServiceClient = (
        client.get_service("CampaignAssetSetService")
    )
    response: MutateCampaignAssetSetsResponse = (
        campaign_asset_set_service.mutate_campaign_asset_sets(
            customer_id=customer_id, operations=[operation]
        )
    )

    resource_name: str = response.results[0].resource_name
    print(f"Created a campaign asset set with resource name: '{resource_name}'")
    # [END add_campaign_asset_set]


# [START add_dsa_target]
def add_dsa_target(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    dsa_page_url_label: str,
) -> None:
    """Creates an ad group criterion targeting the DSA label.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        ad_group_id: the ID for an Ad Group.
        dsa_page_url_label: the label for the DSA page URLs.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    # Creates the ad group criterion.
    operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )
    criterion: AdGroupCriterion = operation.create
    criterion.ad_group = googleads_service.ad_group_path(
        customer_id, ad_group_id
    )
    criterion.cpc_bid_micros = 1500000
    # Creates the webpage info, or criterion for targeting webpages of an
    # advertiser's website.
    criterion.webpage.criterion_name = "Test Criterion"
    # Creates the webpage condition info that targets an advertiser's webpages
    # based on the custom label specified by the DSA page URL label
    # (e.g. "discounts").
    webpage_condition: WebpageConditionInfo = client.get_type(
        "WebpageConditionInfo"
    )
    webpage_condition.operand = (
        client.enums.WebpageConditionOperandEnum.CUSTOM_LABEL
    )
    webpage_condition.argument = dsa_page_url_label
    criterion.webpage.conditions.append(webpage_condition)

    # Issues a mutate request to add the ad group criterion and prints its
    # information.
    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )
    response: MutateAdGroupCriteriaResponse = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id, operations=[operation]
        )
    )
    print(
        "Created ad group criterion with resource name: "
        f"'{response.results[0].resource_name}'"
    )
    # [END add_dsa_target]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Adds a page feed with URLs for a Dynamic Search Ads campaign"
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )

    parser.add_argument(
        "-i",
        "--campaign_id",
        type=str,
        required=True,
        help="The Google Ads campaign ID.",
    )

    parser.add_argument(
        "-a", "--ad_group_id", type=str, help="The Google Ads ad group ID."
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.campaign_id,
            args.ad_group_id,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This code example sends an invitation email to a user.

The invitation is to manage a customer account with a desired access role.
"""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.customer_user_access_invitation_service.client import (
    CustomerUserAccessInvitationServiceClient,
)
from google.ads.googleads.v22.services.types.customer_user_access_invitation_service import (
    CustomerUserAccessInvitationOperation,
    MutateCustomerUserAccessInvitationResponse,
)
from google.ads.googleads.v22.resources.types.customer_user_access_invitation import (
    CustomerUserAccessInvitation,
)

# AccessRoleEnum is part of google.ads.googleads.v22.enums.types.access_role
# but it's accessed via client.enums.AccessRoleEnum, so direct import for type hint might not be strictly needed for the parameter.
# The field invitation.access_role expects an int (the enum value).


def main(
    client: GoogleAdsClient,
    customer_id: str,
    email_address: str,
    access_role: str,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The client customer ID str.
        email_address: The email address for the user receiving the invitation.
        access_role: The desired access role for the invitee (e.g., "ADMIN", "STANDARD").
    """
    service: CustomerUserAccessInvitationServiceClient = client.get_service(
        "CustomerUserAccessInvitationService"
    )
    # [START invite_user_with_access_role]
    invitation_operation: CustomerUserAccessInvitationOperation = (
        client.get_type("CustomerUserAccessInvitationOperation")
    )
    invitation: CustomerUserAccessInvitation = invitation_operation.create
    invitation.email_address = email_address
    # The access_role field in the CustomerUserAccessInvitation message expects
    # an AccessRoleEnum value (which is an int).
    invitation.access_role = client.enums.AccessRoleEnum[access_role].value

    response: MutateCustomerUserAccessInvitationResponse = (
        service.mutate_customer_user_access_invitation(
            customer_id=customer_id, operation=invitation_operation
        )
    )
    print(
        "Customer user access invitation was sent for "
        f"customer ID: '{customer_id}', "
        f"email address {email_address}, and "
        f"access role {access_role}. The invitation resource name is: "
        f"{response.result.resource_name}"
    )
    # [END invite_user_with_access_role]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Sends an invitation email to a user to manage a customer "
            "account with a desired access role."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-e",
        "--email_address",
        type=str,
        required=True,
        help="The email address of the user to send the invitation to.",
    )
    parser.add_argument(
        "-a",
        "--access_role",
        type=str,
        required=True,
        choices=[e.name for e in googleads_client.enums.AccessRoleEnum],
        help="The updated user access role.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(
            googleads_client,
            args.customer_id,
            args.email_address,
            args.access_role,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdvertisingChannelTypeEnum",
    },
)


class AdvertisingChannelTypeEnum(proto.Message):
    r"""The channel type a campaign may target to serve on."""

    class AdvertisingChannelType(proto.Enum):
        r"""Enum describing the various advertising channel types.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            SEARCH (2):
                Search Network. Includes display bundled, and
                Search+ campaigns.
            DISPLAY (3):
                Google Display Network only.
            SHOPPING (4):
                Shopping campaigns serve on the shopping
                property and on google.com search results.
            HOTEL (5):
                Hotel Ads campaigns.
            VIDEO (6):
                Video campaigns.
            MULTI_CHANNEL (7):
                App Campaigns, and App Campaigns for
                Engagement, that run across multiple channels.
            LOCAL (8):
                Local ads campaigns.
            SMART (9):
                Smart campaigns.
            PERFORMANCE_MAX (10):
                Performance Max campaigns.
            LOCAL_SERVICES (11):
                Local services campaigns.
            TRAVEL (13):
                Travel campaigns.
            DEMAND_GEN (14):
                Demand Gen campaigns.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        SEARCH = 2
        DISPLAY = 3
        SHOPPING = 4
        HOTEL = 5
        VIDEO = 6
        MULTI_CHANNEL = 7
        LOCAL = 8
        SMART = 9
        PERFORMANCE_MAX = 10
        LOCAL_SERVICES = 11
        TRAVEL = 13
        DEMAND_GEN = 14


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example illustrates how to add a campaign.

To get campaigns, run get_campaigns.py.
"""


import argparse
import datetime
import sys
from typing import List
import uuid

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.campaign_budget_service import (
    CampaignBudgetServiceClient,
)
from google.ads.googleads.v22.services.types.campaign_budget_service import (
    CampaignBudgetOperation,
    MutateCampaignBudgetsResponse,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.services.types.campaign_service import (
    CampaignOperation,
    MutateCampaignsResponse,
)
from google.ads.googleads.v22.resources.types.campaign_budget import (
    CampaignBudget,
)
from google.ads.googleads.v22.resources.types.campaign import Campaign


_DATE_FORMAT: str = "%Y%m%d"


def main(client: GoogleAdsClient, customer_id: str) -> None:
    campaign_budget_service: CampaignBudgetServiceClient = client.get_service(
        "CampaignBudgetService"
    )
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )

    # [START add_campaigns]
    # Create a budget, which can be shared by multiple campaigns.
    campaign_budget_operation: CampaignBudgetOperation = client.get_type(
        "CampaignBudgetOperation"
    )
    campaign_budget: CampaignBudget = campaign_budget_operation.create
    campaign_budget.name = f"Interplanetary Budget {uuid.uuid4()}"
    campaign_budget.delivery_method = (
        client.enums.BudgetDeliveryMethodEnum.STANDARD
    )
    campaign_budget.amount_micros = 500000

    # Add budget.
    campaign_budget_response: MutateCampaignBudgetsResponse
    try:
        budget_operations: List[CampaignBudgetOperation] = [
            campaign_budget_operation
        ]
        campaign_budget_response = (
            campaign_budget_service.mutate_campaign_budgets(
                customer_id=customer_id,
                operations=budget_operations,
            )
        )
    except GoogleAdsException as ex:
        handle_googleads_exception(ex)
        # [END add_campaigns]
        # We are exiting in handle_googleads_exception so this return is not
        # strictly necessary, but it makes static analysis happier.
        return

    # [START add_campaigns_1]
    # Create campaign.
    campaign_operation: CampaignOperation = client.get_type("CampaignOperation")
    campaign: Campaign = campaign_operation.create
    campaign.name = f"Interplanetary Cruise {uuid.uuid4()}"
    campaign.advertising_channel_type = (
        client.enums.AdvertisingChannelTypeEnum.SEARCH
    )

    # Recommendation: Set the campaign to PAUSED when creating it to prevent
    # the ads from immediately serving. Set to ENABLED once you've added
    # targeting and the ads are ready to serve.
    campaign.status = client.enums.CampaignStatusEnum.PAUSED

    # Set the bidding strategy and budget.
    campaign.manual_cpc = client.get_type("ManualCpc")
    campaign.campaign_budget = campaign_budget_response.results[0].resource_name

    # Set the campaign network options.
    campaign.network_settings.target_google_search = True
    campaign.network_settings.target_search_network = True
    campaign.network_settings.target_partner_search_network = False
    # Enable Display Expansion on Search campaigns. For more details see:
    # https://support.google.com/google-ads/answer/7193800
    campaign.network_settings.target_content_network = True

    # Declare whether or not this campaign serves political ads targeting the
    # EU. Valid values are:
    #   CONTAINS_EU_POLITICAL_ADVERTISING
    #   DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    campaign.contains_eu_political_advertising = (
        client.enums.EuPoliticalAdvertisingStatusEnum.DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    )

    # Optional: Set the start date.
    start_time: datetime.date = datetime.date.today() + datetime.timedelta(
        days=1
    )
    campaign.start_date = datetime.date.strftime(start_time, _DATE_FORMAT)

    # Optional: Set the end date.
    end_time: datetime.date = start_time + datetime.timedelta(weeks=4)
    campaign.end_date = datetime.date.strftime(end_time, _DATE_FORMAT)
    # [END add_campaigns_1]

    # Add the campaign.
    campaign_response: MutateCampaignsResponse
    try:
        campaign_operations: List[CampaignOperation] = [campaign_operation]
        campaign_response = campaign_service.mutate_campaigns(
            customer_id=customer_id, operations=campaign_operations
        )
        print(f"Created campaign {campaign_response.results[0].resource_name}.")
    except GoogleAdsException as ex:
        handle_googleads_exception(ex)


def handle_googleads_exception(exception: GoogleAdsException) -> None:
    print(
        f'Request with ID "{exception.request_id}" failed with status '
        f'"{exception.error.code().name}" and includes the following errors:'
    )
    for error in exception.failure.errors:
        print(f'\tError with message "{error.message}".')
        if error.location:
            for field_path_element in error.location.field_path_elements:
                print(f"\t\tOn field: {field_path_element.field_name}")
    sys.exit(1)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Adds a campaign for specified customer."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    main(googleads_client, args.customer_id)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "BrandSafetySuitabilityEnum",
    },
)


class BrandSafetySuitabilityEnum(proto.Message):
    r"""Container for enum with 3-Tier brand safety suitability
    control.

    """

    class BrandSafetySuitability(proto.Enum):
        r"""3-Tier brand safety suitability control.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            EXPANDED_INVENTORY (2):
                This option lets you show ads across all
                inventory on YouTube and video partners that
                meet our standards for monetization. This option
                may be an appropriate choice for brands that
                want maximum access to the full breadth of
                videos eligible for ads, including, for example,
                videos that have strong profanity in the context
                of comedy or a documentary, or excessive
                violence as featured in video games.
            STANDARD_INVENTORY (3):
                This option lets you show ads across a wide
                range of content that's appropriate for most
                brands, such as popular music videos,
                documentaries, and movie trailers. The content
                you can show ads on is based on YouTube's
                advertiser-friendly content guidelines that take
                into account, for example, the strength or
                frequency of profanity, or the appropriateness
                of subject matter like sensitive events. Ads
                won't show, for example, on content with
                repeated strong profanity, strong sexual
                content, or graphic violence.
            LIMITED_INVENTORY (4):
                This option lets you show ads on a reduced
                range of content that's appropriate for brands
                with particularly strict guidelines around
                inappropriate language and sexual
                suggestiveness; above and beyond what YouTube's
                advertiser-friendly content guidelines address.
                The videos accessible in this sensitive category
                meet heightened requirements, especially for
                inappropriate language and sexual
                suggestiveness. For example, your ads will be
                excluded from showing on some of YouTube's most
                popular music videos and other pop culture
                content across YouTube and Google video
                partners.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        EXPANDED_INVENTORY = 2
        STANDARD_INVENTORY = 3
        LIMITED_INVENTORY = 4


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "UserIdentifierSourceEnum",
    },
)


class UserIdentifierSourceEnum(proto.Message):
    r"""Container for enum describing the source of the user
    identifier for offline Store Sales, click conversion, and
    conversion adjustment uploads.

    """

    class UserIdentifierSource(proto.Enum):
        r"""The type of user identifier source for offline Store Sales,
        click conversion, and conversion adjustment uploads.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version
            FIRST_PARTY (2):
                Indicates that the user identifier was
                provided by the first party (advertiser).
            THIRD_PARTY (3):
                Indicates that the user identifier was
                provided by the third party (partner).
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        FIRST_PARTY = 2
        THIRD_PARTY = 3


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    local_services_lead_credit_issuance_decision,
)
from google.ads.googleads.v22.enums.types import (
    local_services_lead_survey_answer,
)
from google.ads.googleads.v22.enums.types import (
    local_services_lead_survey_dissatisfied_reason,
)
from google.ads.googleads.v22.enums.types import (
    local_services_lead_survey_satisfied_reason,
)
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "AppendLeadConversationRequest",
        "AppendLeadConversationResponse",
        "Conversation",
        "ConversationOrError",
        "SurveySatisfied",
        "SurveyDissatisfied",
        "ProvideLeadFeedbackRequest",
        "ProvideLeadFeedbackResponse",
    },
)


class AppendLeadConversationRequest(proto.Message):
    r"""Request message for
    [LocalServicesLeadService.AppendLeadConversation][google.ads.googleads.v22.services.LocalServicesLeadService.AppendLeadConversation].

    Attributes:
        customer_id (str):
            Required. The Id of the customer which owns
            the leads onto which the conversations will be
            appended.
        conversations (MutableSequence[google.ads.googleads.v22.services.types.Conversation]):
            Required. Conversations that are being
            appended.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    conversations: MutableSequence["Conversation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="Conversation",
    )


class AppendLeadConversationResponse(proto.Message):
    r"""Response message for
    [LocalServicesLeadService.AppendLeadConversation][google.ads.googleads.v22.services.LocalServicesLeadService.AppendLeadConversation].

    Attributes:
        responses (MutableSequence[google.ads.googleads.v22.services.types.ConversationOrError]):
            Required. List of append conversation
            operation results.
    """

    responses: MutableSequence["ConversationOrError"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="ConversationOrError",
    )


class Conversation(proto.Message):
    r"""Details of the conversation that needs to be appended.

    Attributes:
        local_services_lead (str):
            Required. The resource name of the local
            services lead that the conversation should be
            applied to.
        text (str):
            Required. Text message that user wanted to
            append to lead.
    """

    local_services_lead: str = proto.Field(
        proto.STRING,
        number=1,
    )
    text: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ConversationOrError(proto.Message):
    r"""Result of the append conversation operation.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        local_services_lead_conversation (str):
            The resource name of the appended local
            services lead conversation.

            This field is a member of `oneof`_ ``append_lead_conversation_response``.
        partial_failure_error (google.rpc.status_pb2.Status):
            Failure status when the request could not be
            processed.

            This field is a member of `oneof`_ ``append_lead_conversation_response``.
    """

    local_services_lead_conversation: str = proto.Field(
        proto.STRING,
        number=1,
        oneof="append_lead_conversation_response",
    )
    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="append_lead_conversation_response",
        message=status_pb2.Status,
    )


class SurveySatisfied(proto.Message):
    r"""Details about various factors for being satisfied with the
    lead.

    Attributes:
        survey_satisfied_reason (google.ads.googleads.v22.enums.types.LocalServicesLeadSurveySatisfiedReasonEnum.SurveySatisfiedReason):
            Required. Provider's reason for being
            satisfied with the lead.
        other_reason_comment (str):
            Optional. Provider's free form comments. This field is
            required when OTHER_SATISFIED_REASON is selected as the
            reason.
    """

    survey_satisfied_reason: (
        local_services_lead_survey_satisfied_reason.LocalServicesLeadSurveySatisfiedReasonEnum.SurveySatisfiedReason
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=local_services_lead_survey_satisfied_reason.LocalServicesLeadSurveySatisfiedReasonEnum.SurveySatisfiedReason,
    )
    other_reason_comment: str = proto.Field(
        proto.STRING,
        number=2,
    )


class SurveyDissatisfied(proto.Message):
    r"""Details about various factors for not being satisfied with
    the lead.

    Attributes:
        survey_dissatisfied_reason (google.ads.googleads.v22.enums.types.LocalServicesLeadSurveyDissatisfiedReasonEnum.SurveyDissatisfiedReason):
            Required. Provider's reason for not being
            satisfied with the lead.
        other_reason_comment (str):
            Optional. Provider's free form comments. This field is
            required when OTHER_DISSATISFIED_REASON is selected as the
            reason.
    """

    survey_dissatisfied_reason: (
        local_services_lead_survey_dissatisfied_reason.LocalServicesLeadSurveyDissatisfiedReasonEnum.SurveyDissatisfiedReason
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=local_services_lead_survey_dissatisfied_reason.LocalServicesLeadSurveyDissatisfiedReasonEnum.SurveyDissatisfiedReason,
    )
    other_reason_comment: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ProvideLeadFeedbackRequest(proto.Message):
    r"""Request message for
    [LocalServicesLeadService.ProvideLeadFeedback][google.ads.googleads.v22.services.LocalServicesLeadService.ProvideLeadFeedback].

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Required. The resource name of the local
            services lead that for which the feedback is
            being provided.
        survey_answer (google.ads.googleads.v22.enums.types.LocalServicesLeadSurveyAnswerEnum.SurveyAnswer):
            Required. Survey answer for Local Services
            Ads Lead.
        survey_satisfied (google.ads.googleads.v22.services.types.SurveySatisfied):
            Details about various factors for being
            satisfied with the lead.

            This field is a member of `oneof`_ ``survey_details``.
        survey_dissatisfied (google.ads.googleads.v22.services.types.SurveyDissatisfied):
            Details about various factors for not being
            satisfied with the lead.

            This field is a member of `oneof`_ ``survey_details``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    survey_answer: (
        local_services_lead_survey_answer.LocalServicesLeadSurveyAnswerEnum.SurveyAnswer
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=local_services_lead_survey_answer.LocalServicesLeadSurveyAnswerEnum.SurveyAnswer,
    )
    survey_satisfied: "SurveySatisfied" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="survey_details",
        message="SurveySatisfied",
    )
    survey_dissatisfied: "SurveyDissatisfied" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="survey_details",
        message="SurveyDissatisfied",
    )


class ProvideLeadFeedbackResponse(proto.Message):
    r"""Response message for
    [LocalServicesLeadService.ProvideLeadFeedback][google.ads.googleads.v22.services.LocalServicesLeadService.ProvideLeadFeedback].

    Attributes:
        credit_issuance_decision (google.ads.googleads.v22.enums.types.LocalServicesLeadCreditIssuanceDecisionEnum.CreditIssuanceDecision):
            Required. Decision of bonus credit issued or
            rejected. If a bonus credit is issued, it will
            be available for use in about two months.
    """

    credit_issuance_decision: (
        local_services_lead_credit_issuance_decision.LocalServicesLeadCreditIssuanceDecisionEnum.CreditIssuanceDecision
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=local_services_lead_credit_issuance_decision.LocalServicesLeadCreditIssuanceDecisionEnum.CreditIssuanceDecision,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomerPayPerConversionEligibilityFailureReasonEnum",
    },
)


class CustomerPayPerConversionEligibilityFailureReasonEnum(proto.Message):
    r"""Container for enum describing reasons why a customer is not
    eligible to use PaymentMode.CONVERSIONS.

    """

    class CustomerPayPerConversionEligibilityFailureReason(proto.Enum):
        r"""Enum describing possible reasons a customer is not eligible
        to use PaymentMode.CONVERSIONS.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            NOT_ENOUGH_CONVERSIONS (2):
                Customer does not have enough conversions.
            CONVERSION_LAG_TOO_HIGH (3):
                Customer's conversion lag is too high.
            HAS_CAMPAIGN_WITH_SHARED_BUDGET (4):
                Customer uses shared budgets.
            HAS_UPLOAD_CLICKS_CONVERSION (5):
                Customer has conversions with
                ConversionActionType.UPLOAD_CLICKS.
            AVERAGE_DAILY_SPEND_TOO_HIGH (6):
                Customer's average daily spend is too high.
            ANALYSIS_NOT_COMPLETE (7):
                Customer's eligibility has not yet been
                calculated by the Google Ads backend. Check back
                soon.
            OTHER (8):
                Customer is not eligible due to other
                reasons.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NOT_ENOUGH_CONVERSIONS = 2
        CONVERSION_LAG_TOO_HIGH = 3
        HAS_CAMPAIGN_WITH_SHARED_BUDGET = 4
        HAS_UPLOAD_CLICKS_CONVERSION = 5
        AVERAGE_DAILY_SPEND_TOO_HIGH = 6
        ANALYSIS_NOT_COMPLETE = 7
        OTHER = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomerClientLinkErrorEnum",
    },
)


class CustomerClientLinkErrorEnum(proto.Message):
    r"""Container for enum describing possible CustomeClientLink
    errors.

    """

    class CustomerClientLinkError(proto.Enum):
        r"""Enum describing possible CustomerClientLink errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CLIENT_ALREADY_INVITED_BY_THIS_MANAGER (2):
                Trying to manage a client that already in
                being managed by customer.
            CLIENT_ALREADY_MANAGED_IN_HIERARCHY (3):
                Already managed by some other manager in the
                hierarchy.
            CYCLIC_LINK_NOT_ALLOWED (4):
                Attempt to create a cycle in the hierarchy.
            CUSTOMER_HAS_TOO_MANY_ACCOUNTS (5):
                Managed accounts has the maximum number of
                linked accounts.
            CLIENT_HAS_TOO_MANY_INVITATIONS (6):
                Invitor has the maximum pending invitations.
            CANNOT_HIDE_OR_UNHIDE_MANAGER_ACCOUNTS (7):
                Attempt to change hidden status of a link
                that is not active.
            CUSTOMER_HAS_TOO_MANY_ACCOUNTS_AT_MANAGER (8):
                Parent manager account has the maximum number
                of linked accounts.
            CLIENT_HAS_TOO_MANY_MANAGERS (9):
                Client has too many managers.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CLIENT_ALREADY_INVITED_BY_THIS_MANAGER = 2
        CLIENT_ALREADY_MANAGED_IN_HIERARCHY = 3
        CYCLIC_LINK_NOT_ALLOWED = 4
        CUSTOMER_HAS_TOO_MANY_ACCOUNTS = 5
        CLIENT_HAS_TOO_MANY_INVITATIONS = 6
        CANNOT_HIDE_OR_UNHIDE_MANAGER_ACCOUNTS = 7
        CUSTOMER_HAS_TOO_MANY_ACCOUNTS_AT_MANAGER = 8
        CLIENT_HAS_TOO_MANY_MANAGERS = 9


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "OfflineUserDataJobStatusEnum",
    },
)


class OfflineUserDataJobStatusEnum(proto.Message):
    r"""Container for enum describing status of an offline user data
    job.

    """

    class OfflineUserDataJobStatus(proto.Enum):
        r"""The status of an offline user data job.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            PENDING (2):
                The job has been successfully created and
                pending for uploading.
            RUNNING (3):
                Upload(s) have been accepted and data is
                being processed.
            SUCCESS (4):
                Uploaded data has been successfully
                processed.
            FAILED (5):
                Uploaded data has failed to be processed.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PENDING = 2
        RUNNING = 3
        SUCCESS = 4
        FAILED = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "LocalServicesLeadStatusEnum",
    },
)


class LocalServicesLeadStatusEnum(proto.Message):
    r"""Container for enum describing possible statuses of lead."""

    class LeadStatus(proto.Enum):
        r"""Possible statuses of lead.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            NEW (2):
                New lead which hasn't yet been seen by
                advertiser.
            ACTIVE (3):
                Lead that thas been interacted by advertiser.
            BOOKED (4):
                Lead has been booked.
            DECLINED (5):
                Lead was declined by advertiser.
            EXPIRED (6):
                Lead has expired due to inactivity.
            DISABLED (7):
                Disabled due to spam or dispute.
            CONSUMER_DECLINED (8):
                Consumer declined the lead.
            WIPED_OUT (9):
                Personally Identifiable Information of the
                lead is wiped out.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NEW = 2
        ACTIVE = 3
        BOOKED = 4
        DECLINED = 5
        EXPIRED = 6
        DISABLED = 7
        CONSUMER_DECLINED = 8
        WIPED_OUT = 9


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "CampaignExperimentErrorEnum",
    },
)


class CampaignExperimentErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign experiment
    errors.

    """

    class CampaignExperimentError(proto.Enum):
        r"""Enum describing possible campaign experiment errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            DUPLICATE_NAME (2):
                An active campaign or experiment with this
                name already exists.
            INVALID_TRANSITION (3):
                Experiment cannot be updated from the current
                state to the requested target state. For
                example, an experiment can only graduate if its
                status is ENABLED.
            CANNOT_CREATE_EXPERIMENT_WITH_SHARED_BUDGET (4):
                Cannot create an experiment from a campaign
                using an explicitly shared budget.
            CANNOT_CREATE_EXPERIMENT_FOR_REMOVED_BASE_CAMPAIGN (5):
                Cannot create an experiment for a removed
                base campaign.
            CANNOT_CREATE_EXPERIMENT_FOR_NON_PROPOSED_DRAFT (6):
                Cannot create an experiment from a draft,
                which has a status other than proposed.
            CUSTOMER_CANNOT_CREATE_EXPERIMENT (7):
                This customer is not allowed to create an
                experiment.
            CAMPAIGN_CANNOT_CREATE_EXPERIMENT (8):
                This campaign is not allowed to create an
                experiment.
            EXPERIMENT_DURATIONS_MUST_NOT_OVERLAP (9):
                Trying to set an experiment duration which
                overlaps with another experiment.
            EXPERIMENT_DURATION_MUST_BE_WITHIN_CAMPAIGN_DURATION (10):
                All non-removed experiments must start and
                end within their campaign's duration.
            CANNOT_MUTATE_EXPERIMENT_DUE_TO_STATUS (11):
                The experiment cannot be modified because its
                status is in a terminal state, such as REMOVED.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DUPLICATE_NAME = 2
        INVALID_TRANSITION = 3
        CANNOT_CREATE_EXPERIMENT_WITH_SHARED_BUDGET = 4
        CANNOT_CREATE_EXPERIMENT_FOR_REMOVED_BASE_CAMPAIGN = 5
        CANNOT_CREATE_EXPERIMENT_FOR_NON_PROPOSED_DRAFT = 6
        CUSTOMER_CANNOT_CREATE_EXPERIMENT = 7
        CAMPAIGN_CANNOT_CREATE_EXPERIMENT = 8
        EXPERIMENT_DURATIONS_MUST_NOT_OVERLAP = 9
        EXPERIMENT_DURATION_MUST_BE_WITHIN_CAMPAIGN_DURATION = 10
        CANNOT_MUTATE_EXPERIMENT_DUE_TO_STATUS = 11


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "ChangeStatusErrorEnum",
    },
)


class ChangeStatusErrorEnum(proto.Message):
    r"""Container for enum describing possible change status errors."""

    class ChangeStatusError(proto.Enum):
        r"""Enum describing possible change status errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            START_DATE_TOO_OLD (3):
                The requested start date is too old.
            CHANGE_DATE_RANGE_INFINITE (4):
                The change_status search request must specify a finite range
                filter on last_change_date_time.
            CHANGE_DATE_RANGE_NEGATIVE (5):
                The change status search request has
                specified invalid date time filters that can
                never logically produce any valid results (for
                example, start time after end time).
            LIMIT_NOT_SPECIFIED (6):
                The change_status search request must specify a LIMIT.
            INVALID_LIMIT_CLAUSE (7):
                The LIMIT specified by change_status request should be less
                than or equal to 10K.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        START_DATE_TOO_OLD = 3
        CHANGE_DATE_RANGE_INFINITE = 4
        CHANGE_DATE_RANGE_NEGATIVE = 5
        LIMIT_NOT_SPECIFIED = 6
        INVALID_LIMIT_CLAUSE = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "ExperimentTypeEnum",
    },
)


class ExperimentTypeEnum(proto.Message):
    r"""Container for enum describing the type of experiment."""

    class ExperimentType(proto.Enum):
        r"""The type of the experiment.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            DISPLAY_AND_VIDEO_360 (2):
                This is a DISPLAY_AND_VIDEO_360 experiment.
            AD_VARIATION (3):
                This is an ad variation experiment.
            YOUTUBE_CUSTOM (5):
                A custom experiment consisting of Video
                campaigns.
            DISPLAY_CUSTOM (6):
                A custom experiment consisting of display
                campaigns.
            SEARCH_CUSTOM (7):
                A custom experiment consisting of search
                campaigns.
            DISPLAY_AUTOMATED_BIDDING_STRATEGY (8):
                An experiment that compares bidding
                strategies for display campaigns.
            SEARCH_AUTOMATED_BIDDING_STRATEGY (9):
                An experiment that compares bidding
                strategies for search campaigns.".
            SHOPPING_AUTOMATED_BIDDING_STRATEGY (10):
                An experiment that compares bidding
                strategies for shopping campaigns.
            SMART_MATCHING (11):
                DEPRECATED. A smart matching experiment with
                search campaigns.
            HOTEL_CUSTOM (12):
                A custom experiment consisting of hotel
                campaigns.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DISPLAY_AND_VIDEO_360 = 2
        AD_VARIATION = 3
        YOUTUBE_CUSTOM = 5
        DISPLAY_CUSTOM = 6
        SEARCH_CUSTOM = 7
        DISPLAY_AUTOMATED_BIDDING_STRATEGY = 8
        SEARCH_AUTOMATED_BIDDING_STRATEGY = 9
        SHOPPING_AUTOMATED_BIDDING_STRATEGY = 10
        SMART_MATCHING = 11
        HOTEL_CUSTOM = 12


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "CriterionTypeEnum",
    },
)


class CriterionTypeEnum(proto.Message):
    r"""The possible types of a criterion."""

    class CriterionType(proto.Enum):
        r"""Enum describing possible criterion types.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            KEYWORD (2):
                Keyword, for example, 'mars cruise'.
            PLACEMENT (3):
                Placement, also known as Website, for
                example, 'www.flowers4sale.com'
            MOBILE_APP_CATEGORY (4):
                Mobile application categories to target.
            MOBILE_APPLICATION (5):
                Mobile applications to target.
            DEVICE (6):
                Devices to target.
            LOCATION (7):
                Locations to target.
            LISTING_GROUP (8):
                Listing groups to target.
            AD_SCHEDULE (9):
                Ad Schedule.
            AGE_RANGE (10):
                Age range.
            GENDER (11):
                Gender.
            INCOME_RANGE (12):
                Income Range.
            PARENTAL_STATUS (13):
                Parental status.
            YOUTUBE_VIDEO (14):
                YouTube Video.
            YOUTUBE_CHANNEL (15):
                YouTube Channel.
            USER_LIST (16):
                User list.
            PROXIMITY (17):
                Proximity.
            TOPIC (18):
                A topic target on the display network (for
                example, "Pets & Animals").
            LISTING_SCOPE (19):
                Listing scope to target.
            LANGUAGE (20):
                Language.
            IP_BLOCK (21):
                IpBlock.
            CONTENT_LABEL (22):
                Content Label for category exclusion.
            CARRIER (23):
                Carrier.
            USER_INTEREST (24):
                A category the user is interested in.
            WEBPAGE (25):
                Webpage criterion for dynamic search ads.
            OPERATING_SYSTEM_VERSION (26):
                Operating system version.
            APP_PAYMENT_MODEL (27):
                App payment model.
            MOBILE_DEVICE (28):
                Mobile device.
            CUSTOM_AFFINITY (29):
                Custom affinity.
            CUSTOM_INTENT (30):
                Custom intent.
            LOCATION_GROUP (31):
                Location group.
            CUSTOM_AUDIENCE (32):
                Custom audience
            COMBINED_AUDIENCE (33):
                Combined audience
            KEYWORD_THEME (34):
                Smart Campaign keyword theme
            AUDIENCE (35):
                Audience
            NEGATIVE_KEYWORD_LIST (36):
                Negative Keyword List
            LOCAL_SERVICE_ID (37):
                Local Services Ads Service ID.
            SEARCH_THEME (38):
                Search Theme.
            BRAND (39):
                Brand
            BRAND_LIST (40):
                Brand List
            LIFE_EVENT (41):
                Life Event
            WEBPAGE_LIST (42):
                Webpage List
            VIDEO_LINEUP (43):
                Video lineup
            PLACEMENT_LIST (44):
                Placement List
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        KEYWORD = 2
        PLACEMENT = 3
        MOBILE_APP_CATEGORY = 4
        MOBILE_APPLICATION = 5
        DEVICE = 6
        LOCATION = 7
        LISTING_GROUP = 8
        AD_SCHEDULE = 9
        AGE_RANGE = 10
        GENDER = 11
        INCOME_RANGE = 12
        PARENTAL_STATUS = 13
        YOUTUBE_VIDEO = 14
        YOUTUBE_CHANNEL = 15
        USER_LIST = 16
        PROXIMITY = 17
        TOPIC = 18
        LISTING_SCOPE = 19
        LANGUAGE = 20
        IP_BLOCK = 21
        CONTENT_LABEL = 22
        CARRIER = 23
        USER_INTEREST = 24
        WEBPAGE = 25
        OPERATING_SYSTEM_VERSION = 26
        APP_PAYMENT_MODEL = 27
        MOBILE_DEVICE = 28
        CUSTOM_AFFINITY = 29
        CUSTOM_INTENT = 30
        LOCATION_GROUP = 31
        CUSTOM_AUDIENCE = 32
        COMBINED_AUDIENCE = 33
        KEYWORD_THEME = 34
        AUDIENCE = 35
        NEGATIVE_KEYWORD_LIST = 36
        LOCAL_SERVICE_ID = 37
        SEARCH_THEME = 38
        BRAND = 39
        BRAND_LIST = 40
        LIFE_EVENT = 41
        WEBPAGE_LIST = 42
        VIDEO_LINEUP = 43
        PLACEMENT_LIST = 44


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A set of functions to help load configuration from various locations."""

import functools
import json
import logging.config
import os
import re
import yaml


_logger = logging.getLogger(__name__)


_ENV_PREFIX = "GOOGLE_ADS_"
_REQUIRED_KEYS = ("developer_token", "use_proto_plus")
_OPTIONAL_KEYS = (
    "login_customer_id",
    "endpoint",
    "logging",
    "linked_customer_id",
    "http_proxy",
    "use_cloud_org_for_api_access",
)
_CONFIG_FILE_PATH_KEY = ("configuration_file_path",)
_OAUTH2_INSTALLED_APP_KEYS = ("client_id", "client_secret", "refresh_token")
_OAUTH2_REQUIRED_SERVICE_ACCOUNT_KEYS = ("json_key_file_path",)
_OAUTH2_OPTIONAL_SERVICE_ACCOUNT_KEYS = ("impersonated_email",)
# These keys are deprecated environment variables that can be used in place of
# the primary OAuth2 service account keys for backwards compatibility. They will
# be removed in favor of the primary keys at some point.
_SECONDARY_OAUTH2_SERVICE_ACCOUNT_KEYS = (
    "path_to_private_key_file",
    "delegated_account",
)
_KEYS_ENV_VARIABLES_MAP = {
    key: _ENV_PREFIX + key.upper()
    for key in _REQUIRED_KEYS
    + _OPTIONAL_KEYS
    + _OAUTH2_INSTALLED_APP_KEYS
    + _CONFIG_FILE_PATH_KEY
    + _OAUTH2_REQUIRED_SERVICE_ACCOUNT_KEYS
    + _OAUTH2_OPTIONAL_SERVICE_ACCOUNT_KEYS
    + _SECONDARY_OAUTH2_SERVICE_ACCOUNT_KEYS
}


def _config_validation_decorator(func):
    """A decorator used to easily run validations on configs loaded into dicts.

    Add this decorator to any method that returns the config as a dict.

    Raises:
        ValueError: If the configuration fails validation
    """

    @functools.wraps(func)
    def validation_wrapper(*args, **kwargs):
        config_dict = func(*args, **kwargs)
        validate_dict(config_dict)
        return config_dict

    return validation_wrapper


def _config_parser_decorator(func):
    """A decorator used to easily parse config values.

    Since configs can be loaded from different locations such as env vars or
    from YAML files it's possible that they may have inconsistent types that
    need to be parsed to a different type. Add this decorator to any method
    that returns the config as a dict.
    """

    @functools.wraps(func)
    def parser_wrapper(*args, **kwargs):
        config_dict = func(*args, **kwargs)
        parsed_config = convert_login_customer_id_to_str(config_dict)
        parsed_config = convert_linked_customer_id_to_str(parsed_config)

        config_keys = parsed_config.keys()

        if "logging" in config_keys:
            logging_config = parsed_config["logging"]
            # If the logging config is a dict then it is already in the format
            # that needs to be returned by this method.
            if type(logging_config) is not dict:
                try:
                    parsed_config["logging"] = json.loads(logging_config)
                    # The logger is configured here in case deprecation warnings
                    # need to be logged further down in this method. The logger
                    # is otherwise configured by the GoogleAdsClient class.
                    logging.config.dictConfig(parsed_config["logging"])
                except json.JSONDecodeError:
                    raise ValueError(
                        "Could not configure the client because the logging "
                        "configuration defined in the 'logging' key or "
                        "'GOOGLE_ADS_LOGGING' environment variable is invalid. "
                        "The configuration value should be a valid JSON string."
                    )

        if "path_to_private_key_file" in config_keys:
            _logger.warning(
                "The 'path_to_private_key_file' configuration key and "
                "'GOOGLE_ADS_PATH_TO_PRIVATE_KEY_FILE' environment variable "
                "are deprecated and support will be removed at some point in "
                "the future. Please use 'json_key_file_path' configuration key "
                "or 'GOOGLE_ADS_JSON_KEY_FILE_PATH' environment variable "
                "instead."
            )
            if "json_key_file_path" not in config_keys:
                parsed_config["json_key_file_path"] = parsed_config[
                    "path_to_private_key_file"
                ]

            del parsed_config["path_to_private_key_file"]

        if "delegated_account" in config_keys:
            _logger.warning(
                "The 'delegated_account' configuration key and "
                "'GOOGLE_ADS_DELEGATED_PATH' environment variable are "
                "deprecated and support will be removed at some point in "
                "the future. Please use 'impersonated_email' configuration key "
                "or 'GOOGLE_ADS_IMPERSONATED_EMAIL' environment variable "
                "instead."
            )
            if "impersonated_email" not in config_keys:
                parsed_config["impersonated_email"] = parsed_config[
                    "delegated_account"
                ]

            del parsed_config["delegated_account"]

        if "use_proto_plus" in config_keys:
            # When loaded from YAML, YAML string or a dict, this value is
            # evaluated as a bool. If it's loaded from an environment variable
            # it's evaluated as a string. If set to "False" as an environment
            # variable we need to manually change it to the bool False because
            # the string "False" is truthy and can easily be incorrectly
            # converted to the boolean True.
            value = parsed_config.get("use_proto_plus", False)
            parsed_config["use_proto_plus"] = disambiguate_string_bool(value)

        return parsed_config

    return parser_wrapper


def validate_dict(config_data):
    """Validates the given configuration dict.

    Validations that are performed include:
        1. Ensuring all required keys are present.
        2. If a login_customer_id is present ensure it's valid
        3. If a linked_customer_id is present ensure it's valid

    Args:
        config_data: a dict with configuration data.

    Raises:
        ValueError: If the dict does not contain all required config keys.
    """
    if not "use_proto_plus" in config_data.keys():
        raise ValueError(
            "The client library configuration is missing the required "
            '"use_proto_plus" key. Please set this option to either "True" '
            'or "False". For more information about this option see the '
            "Protobuf Messages guide: "
            "https://developers.google.com/google-ads/api/docs/client-libs/python/protobuf-messages"
        )

    if not all(key in config_data for key in _REQUIRED_KEYS):
        raise ValueError(
            "A required field in the configuration data was not "
            "found. The required fields are: {}".format(str(_REQUIRED_KEYS))
        )

    if "login_customer_id" in config_data:
        validate_login_customer_id(config_data["login_customer_id"])

    if "linked_customer_id" in config_data:
        validate_linked_customer_id(config_data["linked_customer_id"])


def _validate_customer_id(customer_id, id_type):
    """Validates a customer ID.

    Args:
        customer_id: a str from config indicating a login customer ID or
            linked customer ID.
        id_type: a str of the type of customer ID, either "login" or "linked".

    Raises:
        ValueError: If the customer ID is not a str representing a ten-digit
            non-negative integer.
    """
    if customer_id is not None:
        # Checks that the string is comprised only of 10 digits.
        pattern = re.compile(r"^\d{10}", re.ASCII)
        if not pattern.fullmatch(customer_id):
            raise ValueError(
                f"The specified {id_type} customer ID is invalid. It must be a "
                "ten digit number represented as a string, i.e. '1234567890'"
            )


def validate_login_customer_id(login_customer_id):
    """Validates a login customer ID.
    Args:
        login_customer_id: a str from config indicating a login customer ID.
    Raises:
        ValueError: If the login customer ID is not an int in the
            range 0 - 9999999999.
    """
    _validate_customer_id(login_customer_id, "login")


def validate_linked_customer_id(linked_customer_id):
    """Validates a linked customer ID.
    Args:
        linked_customer_id: a str from config indicating a linked customer ID.
    Raises:
        ValueError: If the linked customer ID is not an int in the
            range 0 - 9999999999.
    """
    _validate_customer_id(linked_customer_id, "linked")


@_config_validation_decorator
@_config_parser_decorator
def load_from_yaml_file(path=None):
    """Loads configuration data from a YAML file and returns it as a dict.

    Args:
        path: a str indicating the path to a YAML file containing
            configuration data used to initialize a GoogleAdsClient.

    Returns:
        A dict with configuration from the specified YAML file.

    Raises:
        FileNotFoundError: If the specified configuration file doesn't exist.
        IOError: If the configuration file can't be loaded.
    """
    if path is None:
        # If no path is specified then we check for the environment variable
        # that may define the path. If that is not defined then we use the
        # default path.
        path_from_env_var = os.environ.get(
            _ENV_PREFIX + _CONFIG_FILE_PATH_KEY[0].upper()
        )
        path = (
            path_from_env_var
            if path_from_env_var
            else os.path.join(os.path.expanduser("~"), "google-ads.yaml")
        )

    if not os.path.isabs(path):
        path = os.path.expanduser(path)

    with open(path, "rb") as handle:
        yaml_doc = handle.read()

    return parse_yaml_document_to_dict(yaml_doc)


@_config_validation_decorator
@_config_parser_decorator
def load_from_dict(config_dict):
    """Check if the argument is dictionary or not. If successful it calls the parsing decorator,
    followed by validation decorator. This validates the keys used in the config_dict, before
    returning to its caller.

    Args:
        config_dict: a dict containing client configuration.

    Returns:
        The same input dictionary that is passed into the function.

    Raises:
        A value error if the argument (config_dict) is not a dict.
    """
    if isinstance(config_dict, dict):
        return config_dict
    else:
        raise ValueError(
            "The configuration object passed to function load_from_dict must be of type dict."
        )


@_config_validation_decorator
@_config_parser_decorator
def parse_yaml_document_to_dict(yaml_doc):
    """Parses a YAML document to a dict.

    Args:
        yaml_doc: a str (in Python 2) or bytes (in Python 3) containing YAML
            configuration data.

    Returns:
        A dict of the key/value pairs from the given YAML document.

    Raises:
        yaml.YAMLError: If there is a problem parsing the YAML document.
    """
    return yaml.safe_load(yaml_doc) or {}


@_config_validation_decorator
@_config_parser_decorator
def load_from_env():
    """Loads configuration data from the environment and returns it as a dict.

    Returns:
        A dict with configuration from the environment.

    Raises:
        ValueError: If the configuration
    """
    config_data = {
        key: os.environ.get(env_variable)
        for key, env_variable in _KEYS_ENV_VARIABLES_MAP.items()
        if env_variable in os.environ
    }

    # If configuration_file_path is set by the environment then configuration
    # is retrieved from the yaml file specified in the given path.
    if "configuration_file_path" in config_data.keys():
        return load_from_yaml_file(config_data["configuration_file_path"])

    return config_data


def get_oauth2_installed_app_keys():
    """A getter that returns the required OAuth2 installed application keys.

    Returns:
        A tuple containing the required keys as strs.
    """
    return _OAUTH2_INSTALLED_APP_KEYS


def get_oauth2_required_service_account_keys():
    """A getter that returns the required OAuth2 service account keys.

    Returns:
        A tuple containing the required keys as strs.
    """
    return _OAUTH2_REQUIRED_SERVICE_ACCOUNT_KEYS


def convert_login_customer_id_to_str(config_data):
    """Parses a config dict's login_customer_id attr value to a str.

    Like many values from YAML it's possible for login_customer_id to
    either be a str or an int. Since we actually run validations on this
    value before making requests it's important to parse it to a str.

    Args:
        config_data: A config dict object.

    Returns:
        The same config dict object with a mutated login_customer_id attr.
    """
    login_customer_id = config_data.get("login_customer_id")

    if login_customer_id:
        config_data["login_customer_id"] = str(login_customer_id)

    return config_data


def convert_linked_customer_id_to_str(config_data):
    """Parses a config dict's linked_customer_id attr value to a str.

    Like many values from YAML it's possible for linked_customer_id to
    either be a str or an int. Since we actually run validations on this
    value before making requests it's important to parse it to a str.

    Args:
        config_data: A config dict object.

    Returns:
        The same config dict object with a mutated linked_customer_id attr.
    """
    linked_customer_id = config_data.get("linked_customer_id")

    if linked_customer_id:
        config_data["linked_customer_id"] = str(linked_customer_id)

    return config_data


def disambiguate_string_bool(value):
    """Converts a stringified boolean to its bool representation.

    Args:
        value: A boolean or a string representing a boolean.

    Returns:
        A boolean.

    Raises:
        TypeError, ValueError: If the string is not a valid boolean
            representation.
    """
    # This section reproduces the logic from the now deprecated
    # distutils.util.strtobool. The below values are the same used by strtobool
    # as true/false equivalents.
    true_equivalents = ("y", "yes", "t", "true", "on", "1")
    false_equivalents = ("n", "no", "f", "false", "off", "0")

    if isinstance(value, bool):
        return value
    elif isinstance(value, str):
        if value.lower() in true_equivalents:
            return True
        elif value.lower() in false_equivalents:
            return False
        else:
            raise ValueError(
                "The 'use_proto_plus' configuration key value must be "
                f"explicitly set to {true_equivalents} for 'true', or "
                f"{false_equivalents} for 'false', but '{value}' was given."
            )
    else:
        raise TypeError(
            'The "use_proto_plus" configuration key is invalid. Expected '
            f"Union[bool, str] but received {type(value)}"
        )



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "SearchTermMatchSourceEnum",
    },
)


class SearchTermMatchSourceEnum(proto.Message):
    r"""Container for enum describing the source for search term
    matches in the search term report.

    """

    class SearchTermMatchSource(proto.Enum):
        r"""The search term match sources.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ADVERTISER_PROVIDED_KEYWORD (2):
                The match is from a user-provided keyword.
            AI_MAX_KEYWORDLESS (3):
                The match is from the keywordless expansion
                portion of AI Max.
            AI_MAX_BROAD_MATCH (4):
                The match is from the broad match expansion
                portion of AI Max.
            DYNAMIC_SEARCH_ADS (5):
                The match is from a Dynamic Search Ad.
            PERFORMANCE_MAX (6):
                The match is from the search term matching
                functionality in PMax.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ADVERTISER_PROVIDED_KEYWORD = 2
        AI_MAX_KEYWORDLESS = 3
        AI_MAX_BROAD_MATCH = 4
        DYNAMIC_SEARCH_ADS = 5
        PERFORMANCE_MAX = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "CampaignBudgetErrorEnum",
    },
)


class CampaignBudgetErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign budget
    errors.

    """

    class CampaignBudgetError(proto.Enum):
        r"""Enum describing possible campaign budget errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CAMPAIGN_BUDGET_CANNOT_BE_SHARED (17):
                The campaign budget cannot be shared.
            CAMPAIGN_BUDGET_REMOVED (2):
                The requested campaign budget no longer
                exists.
            CAMPAIGN_BUDGET_IN_USE (3):
                The campaign budget is associated with at
                least one campaign, and so the campaign budget
                cannot be removed.
            CAMPAIGN_BUDGET_PERIOD_NOT_AVAILABLE (4):
                Customer is not on the allow-list for this
                campaign budget period.
            CANNOT_MODIFY_FIELD_OF_IMPLICITLY_SHARED_CAMPAIGN_BUDGET (6):
                This field is not mutable on implicitly
                shared campaign budgets
            CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_IMPLICITLY_SHARED (7):
                Cannot change explicitly shared campaign
                budgets back to implicitly shared ones.
            CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED_WITHOUT_NAME (8):
                An implicit campaign budget without a name
                cannot be changed to explicitly shared campaign
                budget.
            CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED (9):
                Cannot change an implicitly shared campaign
                budget to an explicitly shared one.
            CANNOT_USE_IMPLICITLY_SHARED_CAMPAIGN_BUDGET_WITH_MULTIPLE_CAMPAIGNS (10):
                Only explicitly shared campaign budgets can
                be used with multiple campaigns.
            DUPLICATE_NAME (11):
                A campaign budget with this name already
                exists.
            MONEY_AMOUNT_IN_WRONG_CURRENCY (12):
                A money amount was not in the expected
                currency.
            MONEY_AMOUNT_LESS_THAN_CURRENCY_MINIMUM_CPC (13):
                A money amount was less than the minimum CPC
                for currency.
            MONEY_AMOUNT_TOO_LARGE (14):
                A money amount was greater than the maximum
                allowed.
            NEGATIVE_MONEY_AMOUNT (15):
                A money amount was negative.
            NON_MULTIPLE_OF_MINIMUM_CURRENCY_UNIT (16):
                A money amount was not a multiple of a
                minimum unit.
            TOTAL_BUDGET_AMOUNT_MUST_BE_UNSET_FOR_BUDGET_PERIOD_DAILY (18):
                Total budget amount must be unset when
                BudgetPeriod is DAILY.
            INVALID_PERIOD (19):
                The period of the budget is not allowed.
            CANNOT_USE_ACCELERATED_DELIVERY_MODE (20):
                Cannot use accelerated delivery method on
                this budget.
            BUDGET_AMOUNT_MUST_BE_UNSET_FOR_CUSTOM_BUDGET_PERIOD (21):
                Budget amount must be unset when BudgetPeriod
                is CUSTOM.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CAMPAIGN_BUDGET_CANNOT_BE_SHARED = 17
        CAMPAIGN_BUDGET_REMOVED = 2
        CAMPAIGN_BUDGET_IN_USE = 3
        CAMPAIGN_BUDGET_PERIOD_NOT_AVAILABLE = 4
        CANNOT_MODIFY_FIELD_OF_IMPLICITLY_SHARED_CAMPAIGN_BUDGET = 6
        CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_IMPLICITLY_SHARED = 7
        CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED_WITHOUT_NAME = 8
        CANNOT_UPDATE_CAMPAIGN_BUDGET_TO_EXPLICITLY_SHARED = 9
        CANNOT_USE_IMPLICITLY_SHARED_CAMPAIGN_BUDGET_WITH_MULTIPLE_CAMPAIGNS = (
            10
        )
        DUPLICATE_NAME = 11
        MONEY_AMOUNT_IN_WRONG_CURRENCY = 12
        MONEY_AMOUNT_LESS_THAN_CURRENCY_MINIMUM_CPC = 13
        MONEY_AMOUNT_TOO_LARGE = 14
        NEGATIVE_MONEY_AMOUNT = 15
        NON_MULTIPLE_OF_MINIMUM_CURRENCY_UNIT = 16
        TOTAL_BUDGET_AMOUNT_MUST_BE_UNSET_FOR_BUDGET_PERIOD_DAILY = 18
        INVALID_PERIOD = 19
        CANNOT_USE_ACCELERATED_DELIVERY_MODE = 20
        BUDGET_AMOUNT_MUST_BE_UNSET_FOR_CUSTOM_BUDGET_PERIOD = 21


__all__ = tuple(sorted(__protobuf__.manifest))
